package com.ruoyi.system.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.ruoyi.common.core.domain.TreeSelect;
import com.ruoyi.common.core.domain.entity.SysDept;

import java.util.List;

/**
 * 锅炉管理 服务层
 *
 * @author ruoyi
 */
public interface ISysDeptService extends IService<SysDept>
{
    /**
     * 查询锅炉管理数据
     *
     * @param dept 锅炉信息
     * @return 锅炉信息集合
     */
    public List<SysDept> selectDeptList(SysDept dept);

    /**
     * 构建前端所需要树结构
     *
     * @param depts 锅炉列表
     * @return 树结构列表
     */
    public List<SysDept> buildDeptTree(List<SysDept> depts);

    /**
     * 构建前端所需要下拉树结构
     *
     * @param depts 锅炉列表
     * @return 下拉树结构列表
     */
    public List<TreeSelect> buildDeptTreeSelect(List<SysDept> depts);

    /**
     * 根据角色ID查询锅炉树信息
     *
     * @param roleId 角色ID
     * @return 选中锅炉列表
     */
    public List<Integer> selectDeptListByRoleId(Long roleId);

    /**
     * 根据锅炉ID查询信息
     *
     * @param deptId 锅炉ID
     * @return 锅炉信息
     */
    public SysDept selectDeptById(Long deptId);

    /**
     * 根据ID查询所有子锅炉（正常状态）
     *
     * @param deptId 锅炉ID
     * @return 子锅炉数
     */
    public int selectNormalChildrenDeptById(Long deptId);

    /**
     * 是否存在锅炉子节点
     *
     * @param deptId 锅炉ID
     * @return 结果
     */
    public boolean hasChildByDeptId(Long deptId);

    /**
     * 查询锅炉是否存在用户
     *
     * @param deptId 锅炉ID
     * @return 结果 true 存在 false 不存在
     */
    public boolean checkDeptExistUser(Long deptId);

    /**
     * 校验锅炉名称是否唯一
     *
     * @param dept 锅炉信息
     * @return 结果
     */
    public String checkDeptNameUnique(SysDept dept);

    /**
     * 校验锅炉是否有数据权限
     *
     * @param deptId 锅炉id
     */
    public void checkDeptDataScope(Long deptId);

    /**
     * 新增保存锅炉信息
     *
     * @param dept 锅炉信息
     * @return 结果
     */
    public int insertDept(SysDept dept);

    /**
     * 修改保存锅炉信息
     *
     * @param dept 锅炉信息
     * @return 结果
     */
    public int updateDept(SysDept dept);

    /**
     * 删除锅炉管理信息
     *
     * @param deptId 锅炉ID
     * @return 结果
     */
    public int deleteDeptById(Long deptId);
}
